
import numpy
import scipy.linalg
from apgl.util.Parameter import Parameter
from apgl.util.Util import Util 

"""
An implementation of the Primal PCA algorithm. 
"""

class PrimalPCA(object):
    def __init__(self, k):
        Parameter.checkInt(k, 1, float('inf'))
        self.k = k 


    def learnModel(self, X):
        """
        Learn the PCA directions.
        """
        Parameter.checkClass(X, numpy.ndarray)
        Parameter.checkInt(self.k, 1, Util.rank(X, 10**-6))

        C = numpy.dot(X.T, X)
        (lmbdas, U) = scipy.linalg.eig(C)

        self.lmbdas = lmbdas[0:self.k]
        self.U = U[:, 0:self.k] 

        return self.U, self.lmbdas

    def project(self, testX):
        Parameter.checkClass(testX, numpy.ndarray)
        
        return numpy.dot(testX, self.U)

