'''
Created on 24 Jul 2009

@author: charanpal
'''
import unittest

import logging
from apgl.predictors.MlpySVM import MlpySVM
from apgl.data.ExamplesGenerator import ExamplesGenerator

class MlpySVMTest(unittest.TestCase):
    def setUp(self):
        try:
            import mlpy
            numExamples = 100
            numFeatures = 10
            eg = ExamplesGenerator()

            self.examplesList = eg.generateBinaryExamples(numExamples, numFeatures)
            self.svm = MlpySVM()
        except ImportError:
            print("Mlpy is required to run this test script.")

    def tearDown(self):
        pass

    def testLearnModel(self):
        self.svm.learnModel(self.examplesList)
        predY = self.svm.classify(self.examplesList)
        y = self.examplesList.getDataField(self.examplesList.getLabelsName()).ravel()
        
        e = mlpy.err(predY, y) 
        logging.info("Error rate : "  + str(e))
        
        #Test an empty example list - should raise a value error in SVM code
        numExamples = 0 
        numFeatures = 1 
        eg = ExamplesGenerator()
        
        examplesList = eg.generateBinaryExamples(numExamples, numFeatures)
        self.assertRaises(ValueError, self.svm.learnModel, examplesList)
        
    def testClassify(self):
        self.svm.learnModel(self.examplesList)
        predY = self.svm.classify(self.examplesList)
        y = self.examplesList.getDataField(self.examplesList.getLabelsName()).ravel()
        
        e = mlpy.err(predY, y) 
        
        #Now, permute examples 
        self.examplesList.permuteData()
        predY = self.svm.classify(self.examplesList)
        y = self.examplesList.getSampledDataField(self.examplesList.getLabelsName()).ravel()
        
        e2 = mlpy.err(predY, y) 
        
        self.assertEquals(e, e2)
    
    #Quite a simple test   
    #Followed up by printing and examining intermediate variables in the method   
    def testCvModelSelection(self):
        kernel = "linear"
        kernelParams = [0.1, 0.2, 0.4]
        Cs = [1.0, 0.5, 0.25, 0.125]
        errorCosts = [0.1, 0.5]
        folds = 5 
        
        C, kernelParam, errorCost, error = self.svm.cvModelSelection(self.examplesList,Cs, kernelParams, kernel, folds, errorCosts)
        
        self.assertTrue(C in Cs)
        self.assertTrue(kernelParam in kernelParams)
        self.assertTrue(error >= 0 and error <= 1)
        
        #Test gaussian kernel 
        kernel= "gaussian"
        
        C, kernelParam, errorCost, error = self.svm.cvModelSelection(self.examplesList,Cs, kernelParams, kernel, folds, errorCosts)
        
        self.assertTrue(C in Cs)
        self.assertTrue(kernelParam in kernelParams)
        self.assertTrue(error >= 0 and error <= 1)
    
    def testEvaluateCv(self):
        folds = 10
        (means, vars) = self.svm.evaluateCv(self.examplesList, folds)

        self.assertEquals(means.shape[0], 6)
        self.assertEquals(vars.shape[0], 6)

        self.assertTrue((means <= 1).all())
        self.assertTrue((means>= 0).all())
        self.assertTrue((vars <= 1).all())
        self.assertTrue((vars>= 0).all())


if __name__ == "__main__":
    #import sys;sys.argv = ['', 'Test.testName']
    unittest.main()