import numpy 
from apgl.graph import *
from apgl.viroscopy.model.HIVVertices import HIVVertices

"""
A HIVGraph is a SparseGraph except that its vertices are HIVIndividuals and
there are some useful method to find the S, I and R individuals. 
"""

class HIVGraph(SparseGraph):
    def __init__(self, numVertices, undirected=True, W=None):
        """
        Create a graph with the specified number of vertices, and choose whether
        edges are directed. 
        """
        vList = HIVVertices(numVertices)
        super(HIVGraph, self).__init__(vList, undirected, W)

    def getSusceptibleSet(self):
        V = self.vList.getVertices(range(self.getNumVertices()))
        susceptibleSet = numpy.nonzero(V[:, HIVVertices.stateIndex] == HIVVertices.susceptible)[0]
        return set(susceptibleSet.tolist()) 

    def getInfectedSet(self):
        V = self.vList.getVertices(range(self.getNumVertices()))
        infectedSet = numpy.nonzero(V[:, HIVVertices.stateIndex] == HIVVertices.infected)[0]
        return set(infectedSet.tolist())

    def getRemovedSet(self):
        V = self.vList.getVertices(range(self.getNumVertices()))
        removedSet = numpy.nonzero(V[:, HIVVertices.stateIndex] == HIVVertices.removed)[0]
        return set(removedSet.tolist())

        
        
