
from apgl.graph import *
from apgl.util import *
from apgl.viroscopy.model.HIVGraph import HIVGraph
from apgl.viroscopy.model.HIVEpidemicModel import HIVEpidemicModel
from apgl.viroscopy.model.HIVRates import HIVRates

import scipy 
import logging
import sys
import numpy
import pickle
import math 

"""
This is the epidemic model for the HIV spread in cuba. 
"""

logging.basicConfig(stream=sys.stdout, level=logging.INFO)
numpy.random.seed(21)

#Total number of people in population
M = 10000
#Number of people infected at the start of the population
numInitialInfected = 5
defaultFreeDegree = 50

#The graph is one in which edges represent a contact
undirected = True
W = scipy.sparse.lil_matrix((M, M))
graph = HIVGraph(M, undirected, W=W)

for i in range(M):
    vertex = graph.getVertex(i)

    #Set the infection time of a number of individuals to 0 
    if i < numInitialInfected:
        graph.getVertexList().setInfected(i, 0.0)

logging.info("Created graph: " + str(graph))

#End time in days 
T = 300.0
#Time step in days to record the graph for analysis
recordStep = int(T/20)
rates = HIVRates(graph)
model = HIVEpidemicModel(graph, rates)

times, infectedIndices, removedIndices, graph = model.simulate(T, recordStep)

#Let's just save the graph here
#Could just save the detected individuals to mirror the real data 
outputDir = PathDefaults.getOutputDir() + "viroscopy/"

graphFileName = outputDir + "epidemicGraph.pkl"
graphFile = open(graphFileName, 'wb')
pickle.dump(graph, graphFile)
graphFile.close()
logging.info("Saved epidemic graph to file " + graphFileName)

indicesFileName = outputDir + "epidemicDetectIndices.pkl"
indicesFile = open(indicesFileName, 'wb')
pickle.dump(removedIndices, indicesFile)
indicesFile.close()
logging.info("Saved epidemic detected indices to file " + indicesFileName)

timesFileName = outputDir + "epidemicTimes.pkl"
timesFile = open(timesFileName, 'wb')
pickle.dump(times, timesFile)
timesFile.close()
logging.info("Saved epidemic time points to file " + timesFileName)


#TODO: 
#Inference - can compute which pairs of people are likely to have contact
#and also which ones get infected.