import logging
import sys
import cProfile
import pstats
import numpy
import scipy 
from apgl.viroscopy.model.HIVEpidemicModel import HIVEpidemicModel
from apgl.viroscopy.model.HIVGraph import HIVGraph
from apgl.viroscopy.model.HIVRates import HIVRates
from apgl.graph import *
from apgl.util import * 

logging.basicConfig(stream=sys.stdout, level=logging.INFO)
numpy.random.seed(22)

class HIVEpidemicModelProfile():
    def __init__(self):
        #Total number of people in population
        M = 10000
        numInitialInfected = 5
        defaultFreeDegree = 50

        #The graph is one in which edges represent a contact
        W = scipy.sparse.csr_matrix((M, M))
        #W = scipy.sparse.lil_matrix((M, M))
        undirected = True
        self.graph = HIVGraph(M, undirected, W)

        for i in range(M):
            vertex = self.graph.getVertex(i)

            #Set the infection time of a number of individuals to 0
            if i < numInitialInfected:
                self.graph.getVertexList().setInfected(i, 0.0)

        outputDirectory = PathDefaults.getOutputDir()
        directory = outputDirectory + "test/"
        self.profileFileName = directory + "profile.cprof"

    def profileSimulate(self):
        #End time
        T = 200.0

        rates = HIVRates(self.graph)
        model = HIVEpidemicModel(self.graph, rates)

        cProfile.runctx('model.simulate(T)', globals(), locals(), self.profileFileName)
        stats = pstats.Stats(self.profileFileName)
        stats.strip_dirs().sort_stats("cumulative").print_stats(40)
        stats.strip_dirs().sort_stats("time").print_stats(40)
        #stats.strip_dirs().sort_stats("file").print_stats(20)

profiler = HIVEpidemicModelProfile()
profiler.profileSimulate()

#Need to speed up getVertex (make direct calls to VertexList) 